require 'spec_helper'

RSpec.describe 'DistributionApi::cancel_card' do

  let(:api_key) { 'key' }
  let(:api_secret) { 'secret' }
  let(:api_endpoint) { 'https://distribution.api.wizypay.com/v1' }
  let(:reference) { '85fb584c-72cb-4f35-b307-ac5442ef7153' }
  let(:cancel_request) { stub_request(:post, "#{api_endpoint}/cards/#{reference}/cancel").
      with(headers: { 'Accept'        => 'application/json',
                      'Authorization' => /WIZYPAY #{api_key}:.*/,
                      'Content-Type'  => 'application/x-www-form-urlencoded' }) }
  before do
    DistributionApi::setup(api_key, api_secret, api_endpoint)
  end

  context 'success' do
    it 'returns empty body' do
      request  = cancel_request.to_return(status: 200, body: '')
      response = DistributionApi::cancel_card(reference)
      expect(request).to have_been_made
      expect(response).to be_empty
    end
  end

  context 'failure' do
    context '4XX' do
      it "throws #{RestClient::Exception}" do
        request = cancel_request.to_return(status: 401, body: { errors: { base: ['authentication_failed'] } }.to_json)
        expect do
          DistributionApi::cancel_card(reference)
        end.to raise_error(RestClient::Exception)
        expect(request).to have_been_made
      end
    end

    context 'timeout' do
      it "throws #{RestClient::Exception}" do
        request = cancel_request.to_timeout
        expect do
          DistributionApi::cancel_card(reference)
        end.to raise_error(RestClient::Exception)
        expect(request).to have_been_made
      end
    end
  end
end